@@ -2,8 +2,9 @@
 
 /**
   * SquirrelMail Login Manager Plugin
-  * Copyright (C) 2003-2007 Paul Lesneiwski <paul@squirrelmail.org>
-  * This program is licensed under GPL. See COPYING for details
+  * Copyright (c) 2002-2009 Paul Lesniewski <paul@squirrelmail.org>
+  * Copyright (c) 2001 Ryan Orth <snubber-sm@seahat.com>
+  * Licensed under the GNU GPL. For full terms see the file COPYING.
   *
   * @package plugins
   * @subpackage vlogin
@@ -22,6 +23,10 @@
 function vlogin_check_configuration_do()
 {
 
+   global $useSessionBased, $allVirtualDomainsAreUnderOneHost, 
+          $virtualUserTableDBFlavor, $sq_ignore_http_x_forwarded_headers;
+
+
    // make sure there is some config out there
    //
    if (!load_vlogin_config())
@@ -31,11 +36,120 @@ function vlogin_check_configuration_do()
    }
 
 
-   // TODO: is there a way to check if the multilogin patch has 
-   //       been applied (only needed for SM up to version x.x.x)
-   //       when $useSessionBased is turned on?
-   // TOOD: should we warn that $useSessionBased is recommended? (ESP over SM version x.x.x)
+   // only need to do this pre-1.5.2, as 1.5.2 will make this
+   // check for us automatically
    //
+   if (!check_sm_version(1, 5, 2))
+   {
+
+      // try to find Compatibility, and then that it is v2.0.12+
+      //
+      if (!function_exists('check_plugin_version')
+       || !check_plugin_version('compatibility', 2, 0, 12, TRUE))
+      {
+         do_err('Login Manager (vlogin) plugin requires the Compatibility plugin version 2.0.12+', FALSE);
+         return TRUE;
+      }
+
+   }
+
+
+   // make sure the multilogin patch is applied when
+   // $useSessionBased is turned on by actually reading
+   // the PHP file and grepping it (only need patch until 1.5.1)
+   //
+   if ($useSessionBased)
+   {
+      if (check_sm_version(1, 4, 3) && !check_sm_version(1, 5, 2)) // 1.4.3 doesn't even have this configtest hook anyway
+      {
+         if (!check_file_contents(SM_PATH . 'include/validate.php', 'include_once\(SM_PATH \. \'plugins\/multilogin\/functions.php\'\);\s+multilogin_sqoverride_config_do\('))
+         {
+            do_err('Login Manager (vlogin) plugin is using $useSessionBased, but the patch from the Multilogin plugin has not been applied', FALSE);
+            return TRUE;
+         }
+      }
+
+
+      // for 1.5.2+, just check to make sure Multilogin plugin is also available
+      //
+      else if (check_sm_version(1, 5, 2))
+      {
+         @include_once(SM_PATH . 'plugins/multilogin/functions.php');
+         if (!function_exists('multilogin_sqoverride_config_do'))
+            do_err('Login Manager (vlogin) plugin used with $useSessionBased turned on (in your system, this is highly recommended) requires the presence of the Multilogin plugin.  Please install it, but you do not need to activate it.', FALSE);
+      }
+
+   }
+
+
+   // for anyone running 1.5.2+, $useSessionBased should be turned on
+   // and the multilogin plugin needs to be here...
+   //
+   // for anyone running 1.4.x WITH the Multilogin plugin, not using
+   // $useSessionBased is ridiculous...
+   //
+   else
+   {
+      if (check_sm_version(1, 5, 2))
+         do_err('Login Manager (vlogin) plugin should be using $useSessionBased = 1', FALSE);
+      else if (is_plugin_enabled('multilogin'))
+         do_err('Login Manager (vlogin) plugin should be using $useSessionBased = 1', FALSE);
+   }
+
+
+   // try to check the validity of the regexp for $allVirtualDomainsAreUnderOneHost
+   // although this might be inaccurate, so we won't make it a fatal error
+   //
+   if ($allVirtualDomainsAreUnderOneHost) 
+   {
+
+      if (!check_php_version(4,1)) 
+      {
+         global $HTTP_SERVER_VARS;
+         $_SERVER = $HTTP_SERVER_VARS;
+      }
+
+      if (!$sq_ignore_http_x_forwarded_headers
+       && isset($_SERVER['HTTP_X_FORWARDED_HOST'])
+       && !empty($_SERVER['HTTP_X_FORWARDED_HOST']))
+         $hostname = $_SERVER['HTTP_X_FORWARDED_HOST'];
+      else
+         $hostname = $_SERVER['HTTP_HOST'];
+
+      // old way, static location of domain on end of URI:
+      // preg_match('/[\/]*(.*?)(\/|$)/', $_SERVER['REQUEST_URI'], $matches);
+      // old way, using REQUEST_URI (some PHP versions have bug with REQUEST_URI)
+      // preg_match($allVirtualDomainsAreUnderOneHost, $_SERVER['REQUEST_URI'], $matches);
+      // 
+      // run pattern match against full URI starting after the protocol (http(s)://)
+      //
+      if (!preg_match($allVirtualDomainsAreUnderOneHost, $hostname . (strpos($_SERVER['PHP_SELF'], '/') === 0 ? '' : '/') . $_SERVER['PHP_SELF'] . (empty($_SERVER['QUERY_STRING']) ? '' : (strpos($_SERVER['QUERY_STRING'], '?') === 0 ? '' : '?') . $_SERVER['QUERY_STRING']), $matches))
+         do_err('Login Manager (vlogin) plugin could not parse the domain name out of the current URI using the $allVirtualDomainsAreUnderOneHost setting ($allVirtualDomainsAreUnderOneHost = "' . $allVirtualDomainsAreUnderOneHost . '" and the current URI = ' . $hostname . (strpos($_SERVER['PHP_SELF'], '/') === 0 ? '' : '/') . $_SERVER['PHP_SELF'] . (empty($_SERVER['QUERY_STRING']) ? '' : (strpos($_SERVER['QUERY_STRING'], '?') === 0 ? '' : '?') . $_SERVER['QUERY_STRING']) . ')', FALSE);
+
+   }
+
+
+   // if a Berkeley/dbm style database is being used for virtual user table 
+   // lookups, make sure dba extension is active and handler exists
+   //
+   if (!empty($virtualUserTableDBFlavor))
+   {
+
+      $required_php_extensions = array('dba');
+      $diff = array_diff($required_php_extensions, get_loaded_extensions());
+      if(count($diff))
+      {
+         do_err('Login Manager (vlogin) plugin requires PHP extension "dba"', FALSE);
+         return TRUE;
+      }
+
+      if (!in_array($virtualUserTableDBFlavor, dba_handlers()))
+      {
+         do_err('Login Manager (vlogin) plugin requires dba handler "' . $virtualUserTableDBFlavor . '"', FALSE);
+         return TRUE;
+      }
+
+   }
 
 
    return FALSE;
@@ -54,6 +168,10 @@ function vlogin_check_configuration_do()
   * note that the custom configuration file settings will override
   * any "meta-coniguration" if there are overlapping settings.
   *
+  * All files are loaded from the main SquirrelMail config/config_vlogin
+  * directory first, if found there.  If not, they are loaded from
+  * the Vlogin data/ directory (plugins/vlogin/data).
+  *
   * @return boolean FALSE if no config file is found, otherwise TRUE.
   *
   */
@@ -66,14 +184,16 @@ function load_vlogin_config()
 
    // load meta config
    //
-   @include_once(SM_PATH . 'plugins/vlogin/data/meta_config.php');
+   if (!@include_once(SM_PATH . 'config/config_vlogin/meta_config.php'))
+      @include_once(SM_PATH . 'plugins/vlogin/data/meta_config.php');
 
 
    // strip domain from username only?
    //
    if ($strip_domain_only)
    {
-      include_once(SM_PATH . 'plugins/vlogin/data/meta_config.strip_domain.php');
+      if (!@include_once(SM_PATH . 'config/config_vlogin/meta_config.strip_domain.php'))
+         include_once(SM_PATH . 'plugins/vlogin/data/meta_config.strip_domain.php');
       $config_settings_loaded = TRUE;
    }
 
@@ -82,18 +202,25 @@ function load_vlogin_config()
    //
    if ($add_default_domain_only)
    {
-      include_once(SM_PATH . 'plugins/vlogin/data/meta_config.add_default_domain.php');
+      if (!@include_once(SM_PATH . 'config/config_vlogin/meta_config.add_default_domain.php'))
+         include_once(SM_PATH . 'plugins/vlogin/data/meta_config.add_default_domain.php');
       $config_settings_loaded = TRUE;
    }
 
 
-   // load custom config if present, or load typical settings 
-   // if none found at all
+   // load custom config if present, or load typical (config_default
+   // is currently a copy of config_example_typical) settings if
+   // none found at all
    //
-   if ((!@include_once(SM_PATH . 'plugins/vlogin/data/config.php'))
-    && !$config_settings_loaded)
-      if (!@include_once(SM_PATH . 'plugins/vlogin/data/config.sample.typical.php'))
-         return FALSE;
+   // Note: could code all these if's into one single test with all &&'s
+   //       but it is kept this way for clarity; won't change unless
+   //       someone can prove that there is an associated performance hit
+   //
+   if (!@include_once(SM_PATH . 'config/config_vlogin/config.php'))
+      if (!@include_once(SM_PATH . 'plugins/vlogin/data/config.php'))
+         if (!$config_settings_loaded)
+            if (!@include_once(SM_PATH . 'plugins/vlogin/data/config_default.php'))
+               return FALSE;
 
    return TRUE;
 
@@ -140,21 +267,59 @@ function overrideSmConfig($args, $user='', $hostname='', $hostname_stripped='',
           $replacements, $removeFromFront, $translateHostnameTable, 
           $pathToQmail, $reverseDotSectionOrder,
           $always_prepend, $always_append, $IMAPServerRules, 
-          $SMTPServerRules;
+          $SMTPServerRules, $sq_ignore_http_x_forwarded_headers;
       
   
-  // in SM 1.5.2+, this function is registered on the config_override
-  // hook, but only really needed for the login and logout pages where
-  // the config settings are not already cached and need to be built
-  // from scratch -- all other config_override calls just need to go 
-  // straight to the multilogin config override function
+  // debugging/performance tuning
   //
-  if (check_sm_version(1, 5, 2) && get_current_hook_name($args) == 'config_override'
-   && !(strpos($PHP_SELF, '/login.php') !== FALSE 
-     || strpos($PHP_SELF, '/signout.php') !== FALSE))
+  //$vlogin_override_config_start = microtime();
+
+
+  $current_hook_name = get_current_hook_name($args);
+
+
+  // in SM 1.5.2+, this function is registered on the config_override
+  // (now renamed to prefs_backend) hook, but only really needed for
+  // the login and logout pages where the config settings are not
+  // already cached and need to be built from scratch -- all other
+  // config_override calls just need to go straight to the multilogin
+  // config override function
+  //
+  // also in SM 1.5.2+, the logout hook does not need to run this
+  // function, since the prefs_backend (config_override) hook will
+  // have already run before it
+  //
+  if (check_sm_version(1, 5, 2))
   {
-     multilogin_sqoverride_config_do($args);
-     return;
+     if ($current_hook_name == 'prefs_backend')
+     {
+
+        // logout error can happen on any page; make sure vlogin
+        // is first on that hook so it can set up the proper
+        // config settings therein (apparently, the session is
+        // getting wiped between this hook and logout_error...?)
+        //
+        reposition_plugin_on_hook('vlogin', 'logout_error');
+
+        if (strpos($PHP_SELF, '/login.php') === FALSE 
+         && strpos($PHP_SELF, '/signout.php') === FALSE)
+        {
+           if (!function_exists('multilogin_sqoverride_config_do'))
+              include_once(SM_PATH . 'plugins/multilogin/functions.php');
+           multilogin_sqoverride_config_do($args);
+           return;
+        }
+     }
+  }
+
+
+  // in SM 1.4.x, we don't want to run on the prefs_backend hook
+  // (that's just for config_override in 1.5.x)
+  //
+  else
+  {
+     if ($current_hook_name == 'prefs_backend')
+        return;
   }
 
 
@@ -212,6 +377,20 @@ function overrideSmConfig($args, $user='', $hostname='', $hostname_stripped='',
      load_vlogin_config();
 
 
+     // In SM 1.4.x when using $useSessionBased, the loading_constants
+     // hook is only needed when on the signout page, login page and
+     // redirect page (have to do this here because we cannot load the
+     // vlogin config until just above)
+     //
+     if (!check_sm_version(1, 5, 2)
+      && $current_hook_name == 'loading_constants'
+      && $useSessionBased
+      && strpos($PHP_SELF, '/login.php') === FALSE
+      && strpos($PHP_SELF, '/redirect.php') === FALSE
+      && strpos($PHP_SELF, '/signout.php') === FALSE)
+        return;
+
+
      // prep: get config settings from session if necessary
      //
      if ($useSessionBased)
@@ -221,8 +400,8 @@ function overrideSmConfig($args, $user='', $hostname='', $hostname_stripped='',
 
         // only clean up when first logging in
         //
-        if (strpos($PHP_SELF, 'redirect.php') !== FALSE
-         || strpos($PHP_SELF, 'login.php') !== FALSE)
+        if (strpos($PHP_SELF, '/redirect.php') !== FALSE
+         || strpos($PHP_SELF, '/login.php') !== FALSE)
         {
            sqsession_unregister('config_override');
    
@@ -237,7 +416,9 @@ function overrideSmConfig($args, $user='', $hostname='', $hostname_stripped='',
 
      // grab hostname into local var
      //
-     if (isset($_SERVER['HTTP_X_FORWARDED_HOST']) && !empty($_SERVER['HTTP_X_FORWARDED_HOST']))
+     if (!$sq_ignore_http_x_forwarded_headers
+      && isset($_SERVER['HTTP_X_FORWARDED_HOST'])
+      && !empty($_SERVER['HTTP_X_FORWARDED_HOST']))
         $hostname = $_SERVER['HTTP_X_FORWARDED_HOST'];
      else
         $hostname = $_SERVER['HTTP_HOST'];
@@ -277,18 +458,6 @@ function overrideSmConfig($args, $user='', $hostname='', $hostname_stripped='',
      $orig_hostname_stripped = $hostname_stripped;
    
    
-     // set domain if the $smHostIsDomainThatUserLoggedInWith
-     // flag is on...
-     //
-     if ($smHostIsDomainThatUserLoggedInWith)
-     {
-        if ($useSessionBased)
-           $config_override['domain'] = $hostname_stripped;
-        else
-           $domain = $hostname_stripped; 
-     }
-      
-
      // allow domain given in username to override host
      // if needed - yikes
      //
@@ -310,6 +479,18 @@ function overrideSmConfig($args, $user='', $hostname='', $hostname_stripped='',
      }
 
 
+     // set domain if the $smHostIsDomainThatUserLoggedInWith
+     // flag is on...
+     //
+     if ($smHostIsDomainThatUserLoggedInWith)
+     {
+        if ($useSessionBased)
+           $config_override['domain'] = $hostname_stripped;
+        else
+           $domain = $hostname_stripped; 
+     }
+      
+
      // determine IMAP server address if it should be based 
      // on the user's domain
      //
@@ -466,7 +647,9 @@ function overrideSmConfig($args, $user='', $hostname='', $hostname_stripped='',
 
      // allow inclusion of external configuration files for each domain
      //
-     if (file_exists(SM_PATH . 'plugins/vlogin/data/domains/' . $hostname_stripped . '.vlogin.config.php'))
+     if (file_exists(SM_PATH . 'config/config_vlogin/domains/' . $hostname_stripped . '.vlogin.config.php'))
+        include_once(SM_PATH . 'config/config_vlogin/domains/' . $hostname_stripped . '.vlogin.config.php');
+     else if (file_exists(SM_PATH . 'plugins/vlogin/data/domains/' . $hostname_stripped . '.vlogin.config.php'))
        include_once(SM_PATH . 'plugins/vlogin/data/domains/' . $hostname_stripped . '.vlogin.config.php');
 
   }
@@ -477,11 +660,15 @@ function overrideSmConfig($args, $user='', $hostname='', $hostname_stripped='',
   // override the org_logo and other stuff if we find a match
   //
   // only need to do this stuff when first logging in or on signout
-  // page when session was possibly lost
+  // page or during the "logout error" process when session was 
+  // possibly lost (or if not storing anything in the session, have 
+  // to do this every time)
   //
-  if (!$checkServiceLevel || strpos($PHP_SELF, 'redirect.php') !== FALSE
-   || strpos($PHP_SELF, 'login.php') !== FALSE 
-   || strpos($PHP_SELF, 'signout.php') !== FALSE)
+  if (!$useSessionBased || !$checkServiceLevel 
+   || $current_hook_name == 'logout_error'
+   || strpos($PHP_SELF, '/redirect.php') !== FALSE
+   || strpos($PHP_SELF, '/login.php') !== FALSE 
+   || strpos($PHP_SELF, '/signout.php') !== FALSE)
 //
 // NOTE: the following line will solve PHP 4.3 problems when using
 //       the session_recall patch, however it will also unfortunately
@@ -680,7 +867,7 @@ function overrideSmConfig($args, $user='', $hostname='', $hostname_stripped='',
       
       // exit this loop, unless this is the global default
       // 
-      if( $virtualDomain != '*' ) break;
+      if ($virtualDomain != '*') break;
 
     }
 
@@ -739,6 +926,13 @@ function overrideSmConfig($args, $user='', $hostname='', $hostname_stripped='',
   perUserOverride($user, $args);
 
 
+  // debugging/performance tuning (might be premature if using service
+  // levels due to recursive call of this function about 25 lines above)
+  //
+  //echo 'Vlogin Override Config Execution Time: ' . sm_microtime_diff($vlogin_override_config_start); exit;
+// Notes: I don't see anything big here that is eating performance except perhaps code called out of this function like the per-user override code or service level override code
+
+
 }
 
 
@@ -762,7 +956,8 @@ function determine_user_hostname()
           $chopOffDotSectionsFromLeft, $numberOfDotSections, 
           $checkByExcludeList, $at, $dot, $replacements,
           $removeFromFront, $translateHostnameTable, $pathToQmail, 
-          $reverseDotSectionOrder, $always_prepend, $always_append;
+          $reverseDotSectionOrder, $always_prepend, $always_append,
+          $sq_ignore_http_x_forwarded_headers;
 
 
    // get vlogin config settings
@@ -784,7 +979,9 @@ function determine_user_hostname()
 
    // grab hostname into local var
    //
-   if (isset($_SERVER['HTTP_X_FORWARDED_HOST']) && !empty($_SERVER['HTTP_X_FORWARDED_HOST']))
+   if (!$sq_ignore_http_x_forwarded_headers
+    && isset($_SERVER['HTTP_X_FORWARDED_HOST'])
+    && !empty($_SERVER['HTTP_X_FORWARDED_HOST']))
       $hostname = $_SERVER['HTTP_X_FORWARDED_HOST'];
    else
       $hostname = $_SERVER['HTTP_HOST'];
@@ -931,7 +1128,13 @@ function vlogin_domain_do($args)
          $alwaysAddHostName, $usernameReplacements, $forceLowercase,
          $prefs_are_cached, $prefs_cache, $vlogin_remap_username_completed,
          $dontUseHostNameUserList, $postProcessingReplacement, 
-         $postProcessingPattern;
+         $postProcessingPattern, $virtualUserTableDBFlavor,
+         $override_config_during_login;
+
+
+  // debugging/performance tuning
+  //
+  //$vlogin_domain_start = microtime();
 
 
   // we've already been here.... don't do this twice
@@ -941,25 +1144,33 @@ function vlogin_domain_do($args)
   if ($vlogin_remap_username_completed) return;
 
 
-  // figure out where prefs are stored
-  //
-  if (isset($prefs_dsn) && !empty($prefs_dsn))
-      $prefsInDB = true;
-  else
-      $prefsInDB = false;
-
-
   // get vlogin config settings
   //
   load_vlogin_config();
 
 
 
-  if ($useSessionBased)
+  // in 1.5.2, we know config settings were overridden on
+  // the prefs_backend (config_override) hook before we
+  // get here, so no need to do it again (in 1.4.x, rely
+  // on what is configured in the vlogin config file)
+  //
+  if (check_sm_version(1, 5, 2))
+     $override_config_during_login = FALSE;
+  if ($override_config_during_login && $useSessionBased)
      overrideSmConfig($args);
 
 
 
+  // figure out where prefs are stored
+  //
+  if (isset($prefs_dsn) && !empty($prefs_dsn))
+      $prefsInDB = true;
+  else
+      $prefsInDB = false;
+
+
+
   // what username did the user submit to us?
   //
   // someone reported that they were able to log in with spaces
@@ -1117,7 +1328,16 @@ function vlogin_domain_do($args)
   //
   if (!empty($sendmailVirtualUserTable))
   {
-     $realname = getSendmailVirtualUser($realname, $sendmailVirtualUserTable);
+
+     // if the user table is in database format, lookup is handled differently...
+     //
+     if (!empty($virtualUserTableDBFlavor))
+        $realname = getVirtualUserFromDB($user, $sendmailVirtualUserTable, 
+                                         $virtualUserTableDBFlavor);
+
+     else
+        $realname = getSendmailVirtualUser($realname, $sendmailVirtualUserTable);
+
   }
 
 
@@ -1154,17 +1374,36 @@ function vlogin_domain_do($args)
   $vlogin_remap_username_completed = TRUE;
 
 
+  // debugging/performance tuning
+  //
+  //echo 'Vlogin Domain Execution Time: ' . sm_microtime_diff($vlogin_domain_start); exit;
+
+
   // when in debug mode, just dump out final login name and quit
   //
   if ($vlogin_debug)
   {
 
-     echo '';
-     echo '';
-     echo '<html><body><hr><br>';
-     echo '<h4>Your IMAP login was resolved to:<br><br>';
-     echo $realname;
-     echo '</h4><br><hr>';
+     global $rawLoginIsOutgoingEmailAddress, $appendDomainToOutgoingEmailAddress, $domain;
+
+     echo "\n"
+        . "\n"
+        . '<html><body><br />'
+        . '<hr /><h4>Your original username was:<br /><br />'
+        . $vlogin_original_username
+        . '</h4><hr />'
+        . '<h4>Your IMAP login was resolved to:<br /><br />'
+        . $realname
+        . '</h4><hr />';
+
+     if ($rawLoginIsOutgoingEmailAddress)
+     {
+        echo '<h4>Your SquirrelMail outgoing email address will be set to:<br /><br />'
+           . $vlogin_original_username
+           . ($appendDomainToOutgoingEmailAddress ? $at . $domain : '')
+           . '</h4><hr />';
+     }
+
      // was confusing to people using $dontUseHostName 
      if (!$dontUseHostName) echo '<br>$hostname is ' . $hostname . '<br>';
      echo '<br>PHP_SELF is ' . (isset($_SERVER['PHP_SELF']) ? $_SERVER['PHP_SELF'] : 'not globally accessible')
@@ -1192,15 +1431,20 @@ function vlogin_save_original_do()
    load_vlogin_config();
 
 
-   global $vlogin_original_username, $rawLoginIsOutgoingEmailAddress, 
-          $username, $data_dir;
+   global $vlogin_original_username, $rawLoginIsOutgoingEmailAddress, $domain,
+          $username, $data_dir, $appendDomainToOutgoingEmailAddress, $at;
    sqsession_register($vlogin_original_username, 'vlogin_original_username');
 
 
    // force outgoing email address?
    //
    if ($rawLoginIsOutgoingEmailAddress)
-      setPref($data_dir, $username, 'email_address', $vlogin_original_username);
+   {
+      if ($appendDomainToOutgoingEmailAddress)
+         setPref($data_dir, $username, 'email_address', $vlogin_original_username . $at . $domain);
+      else
+         setPref($data_dir, $username, 'email_address', $vlogin_original_username);
+   }
 
 }
 
@@ -1305,7 +1549,12 @@ function serviceLevelOverride($user)
 
    // override settings if found
    //
-   if (file_exists(SM_PATH . 'plugins/vlogin/data/service_levels/' . $serviceLevel . '.php'))
+   if (file_exists(SM_PATH . 'config/config_vlogin/service_levels/' . $serviceLevel . '.php'))
+   {
+      include_once(SM_PATH . 'config/config_vlogin/service_levels/' . $serviceLevel . '.php');
+      return TRUE;
+   }
+   else if (file_exists(SM_PATH . 'plugins/vlogin/data/service_levels/' . $serviceLevel . '.php'))
    {
       include_once(SM_PATH . 'plugins/vlogin/data/service_levels/' . $serviceLevel . '.php');
       return TRUE;
@@ -1355,7 +1604,11 @@ function deconstructDomainName($hostname, $notPartOfDomainName,
                                $always_prepend, $always_append)
 {
 
-  load_vlogin_config();
+  // woa!  the following line will override all the arguments above!
+  // why was this here?  was it a bug?  no one reported any bugs...?
+  // I am removing it with hopes that that is the right thing to do...
+  //
+  // load_vlogin_config();
 
 
 
@@ -1594,7 +1847,7 @@ function getSendmailVirtualUser($user, $sendmailVirtualUserTable)
    $atPos = strpos($user, $at);
 
 
-   if ($VIRTTABLE = @fopen ($sendmailVirtualUserTable, 'r'))
+   if ($VIRTTABLE = @fopen($sendmailVirtualUserTable, 'r'))
    {
 
       // be ready to use catchall address - prepare domain name
@@ -1693,6 +1946,134 @@ function getSendmailVirtualUser($user, $sendmailVirtualUserTable)
 
 
 
+/**
+  * Remap User Login using a Berkeley/dbm Style Virtual User Table
+  *
+  * Looks up the given username in the given virtual user table
+  * and, if found therein, returns the corresponding lookup value
+  * (otherwise, returns the username as is).
+  *
+  * @param string $user The username to be looked up.
+  * @param string $virtualUserDBTable The system path to the 
+  *                                   lookup database table.
+  * @param string $virtualUserTableDBFlavor The database type,
+  *                                         optionally with ":null"
+  *                                         appended, which indicates
+  *                                         that DB keys and values
+  *                                         all need to have a NULL
+  *                                         character manually appended 
+  *                                         to them.
+  *
+  * @return string The remapped user name.
+  *
+  */
+function getVirtualUserFromDB($user, $virtualUserDBTable, $virtualUserTableDBFlavor)
+{
+
+   global $at, $vlogin_debug;
+
+
+   $atPos = strpos($user, $at);
+   $virtualUserTableDBFlavor = explode(':', $virtualUserTableDBFlavor);
+
+
+   // default to just return username as is
+   //
+   $remappedUsername = $user;
+
+
+
+   if ($VIRTTABLE = @dba_open($sendmailVirtualUserTable, 'r', 
+                              $virtualUserTableDBFlavor[0]))
+   {
+
+
+      // append null to end of key if needed
+      //
+      $key = $user;
+      if (!empty($virtualUserTableDBFlavor[1])) $key .= chr(0);
+
+
+      // check for username in DB
+      //
+      if (dba_exists($key, $VIRTTABLE))
+      {
+
+         $remappedUsername = dba_fetch($key, $VIRTTABLE);
+         if (!empty($virtualUserTableDBFlavor[1])) 
+            $remappedUsername = substr($remappedUsername, 0, -1);
+
+         // take first value if more than one is found
+         //
+         $remappedUsername = explode(',', $remappedUsername);
+         $remappedUsername = $remappedUsername[0];
+
+      }
+
+
+      // user not found?  look for domain catchall instead
+      //
+      else 
+      {
+
+         $domainName = '';
+         if ($atPos !== FALSE)
+            $domainName = substr($user, $atPos);
+
+         // append null to end of key if needed
+         //
+         $key = $domainName;
+         if (!empty($virtualUserTableDBFlavor[1])) $key .= chr(0);
+
+         if (dba_exists($key, $VIRTTABLE))
+         {
+
+            $remappedUsername = dba_fetch($key, $VIRTTABLE);
+            if (!empty($virtualUserTableDBFlavor[1])) 
+               $remappedUsername = substr($remappedUsername, 0, -1);
+
+            // take first value if more than one is found
+            //
+            $remappedUsername = explode(',', $remappedUsername);
+            $remappedUsername = $remappedUsername[0];
+
+            // remove extraneous stuff we don't need off end (begins with plus sign)
+            //
+            $remappedUsername = preg_replace('/\+%\S+\s*$/', '', $remappedUsername);
+
+         }
+
+      }
+
+      dba_close($VIRTTABLE);
+
+   }
+
+
+   // only show bad fopen info when in debug mode
+   //
+   else if ($vlogin_debug)
+   {
+      global $color;
+      plain_error_message(sprintf("Could not open %s.  Make sure the file exists, is of %s format, and that the web server has permission to read it", $sendmailVirtualUserTable, $virtualUserTableDBFlavor), $color);
+   }
+
+
+   // do variable replacement if needed
+   //
+   if ($atPos !== FALSE && strpos($remappedUsername, '%1') !== FALSE)
+   {
+      $usernameOnly = substr($user, 0, $atPos);
+      $remappedUsername = str_replace('%1', $usernameOnly, $remappedUsername);
+   }
+
+
+   return $remappedUsername;
+
+}
+
+
+
 /**
   * Allows control over which options are displayed on options pages
   *
@@ -1700,99 +2081,107 @@ function getSendmailVirtualUser($user, $sendmailVirtualUserTable)
 function vlogin_display_options_do()
 {
 
-   global $config_override, $optpage_data, $notPartOfDomainName, 
-          $chopOffDotSectionsFromRight, $chopOffDotSectionsFromLeft, 
-          $numberOfDotSections, $checkByExcludeList, $at, $dot, 
-          $replacements, $removeFromFront, $translateHostnameTable, 
-          $pathToQmail, $reverseDotSectionOrder, $always_prepend, 
-          $always_append;
+   global $disable_option_pages, $optpage;
 
-   $overrides = array();
-
-
-   // try to get configuration settings from session, if they're there
+   // make sure user isn't trying to access/submit
+   // a disallowed option page
    //
-   if (!isset($config_override))
+   // 0 = Personal Information = personal
+   // 1 = Display Preferences  = display
+   // 2 = Message Highlighting = highlight
+   // 3 = Folder Preferences   = folder
+   // 4 = Index Order          = order
+   // 5 = Compose Preferences  = compose
+   //
+   if (isset($disable_option_pages) && is_array($disable_option_pages))
    {
 
-      global $virtualDomains, $useSessionBased;
-      load_vlogin_config();
-
-      if (!$useSessionBased)
+      switch ($optpage)
       {
-
-         // get global variable for versions of PHP < 4.1 
-         // doing this instead of sqGetGlobalVar() because
-         // this code might be called before that function
-         // is defined (TODO: as of 1.5.2, this may no longer
-         // be true)
-         //
-         if (!check_php_version(4,1)) 
-         {
-            global $HTTP_SERVER_VARS;
-            $_SERVER = $HTTP_SERVER_VARS;
-         }
-
-
-         // grab hostname into local var
-         //
-         if (isset($_SERVER['HTTP_X_FORWARDED_HOST']) && !empty($_SERVER['HTTP_X_FORWARDED_HOST']))
-            $hostname = $_SERVER['HTTP_X_FORWARDED_HOST'];
-         else
-            $hostname = $_SERVER['HTTP_HOST'];
-         $hostname_stripped = deconstructDomainName($hostname,
-                                                    $notPartOfDomainName, 
-                                                    $chopOffDotSectionsFromRight, 
-                                                    $chopOffDotSectionsFromLeft, 
-                                                    $numberOfDotSections, 
-                                                    $checkByExcludeList, 
-                                                    $at, $dot, $replacements,
-                                                    $removeFromFront, 
-                                                    $translateHostnameTable, 
-                                                    $pathToQmail,
-                                                    $reverseDotSectionOrder,
-                                                    $always_prepend, $always_append);
-
-
-         // allow inclusion of external configuration files for each domain
-         //
-         if (file_exists(SM_PATH . 'plugins/vlogin/data/domains/' . $hostname_stripped . '.vlogin.config.php'))
-            include_once(SM_PATH . 'plugins/vlogin/data/domains/' . $hostname_stripped . '.vlogin.config.php');
-
-
-         // get virtual domain config items
-         //
-         $overrides = array();
-         if (is_array($virtualDomains))
-         foreach (array_keys($virtualDomains) as $virtualDomain) 
-         {
-
-            if (stristr($hostname, (string)($virtualDomain)) || $virtualDomain == '*') 
-            {
-
-               $overrides = $virtualDomains[$virtualDomain];
-
-            }
-
-         }
-
+         case 'personal':
+            if (in_array(0, $disable_option_pages))
+               exit;  // don't print anything out to help attackers
+            break;
+         case 'display':
+            if (in_array(1, $disable_option_pages))
+               exit;  // don't print anything out to help attackers
+            break;
+         case 'highlight':
+            if (in_array(2, $disable_option_pages))
+               exit;  // don't print anything out to help attackers
+            break;
+         case 'folder':
+            if (in_array(3, $disable_option_pages))
+               exit;  // don't print anything out to help attackers
+            break;
+         case 'order':
+            if (in_array(4, $disable_option_pages))
+               exit;  // don't print anything out to help attackers
+            break;
+         case 'compose':
+            if (in_array(5, $disable_option_pages))
+               exit;  // don't print anything out to help attackers
+            break;
       }
 
    }
-   else
-      $overrides = $config_override;
-
 
+   global $disable_options, $optpage_data;
 
    // remove any options from list of options as desired
    //
-   if (isset($overrides['disable_options']) && is_array($overrides['disable_options']))
-      foreach ($overrides['disable_options'] as $removeOptionName)
+   if (isset($disable_options) && is_array($disable_options))
+   {
+      foreach ($disable_options as $removeOptionName)
          foreach ($optpage_data['vals'] as $groupNumber => $optionGroup)
             foreach ($optionGroup as $optionNumber => $optionItem)
                if (isset($optionItem['name']) && $optionItem['name'] == $removeOptionName)
                   unset($optpage_data['vals'][$groupNumber][$optionNumber]);
 
+
+      // check if no other elements are left in the option group
+      // (except hidden elements)...  remove the whole group if
+      // so (are there any hidden elements that we WANT to retain??)
+      //
+      foreach ($optpage_data['vals'] as $groupNumber => $optionGroup)
+      {
+         $group_is_empty = TRUE;
+         foreach ($optionGroup as $optionNumber => $optionItem)
+            if (isset($optionItem['type']) && $optionItem['type'] != SMOPT_TYPE_HIDDEN)
+            {
+               $group_is_empty = FALSE;
+               break;
+            }
+         if ($group_is_empty)
+         {
+            unset($optpage_data['vals'][$groupNumber]);
+            unset($optpage_data['grps'][$groupNumber]);
+         }
+      }
+   }
+
+}
+
+
+
+/**
+  * Allows control over what option blocks (pages)
+  * are available on main options page
+  *
+  */
+function vlogin_option_block_do()
+{
+
+   global $disable_option_pages, $optpage_blocks;
+
+   // remove any option blocks as needed
+   //
+   if (isset($disable_option_pages) && is_array($disable_option_pages))
+   {
+      foreach ($disable_option_pages as $page)
+         unset($optpage_blocks[$page]);
+   }
+
 }
 
 
